@@ -71,7 +71,7 @@ func ParseProfile(binary string, path string) ProfileLines {
 	return out
 }
 
-var _ = Describe("Profiling Specs", func() {
+var _ = Describe("Profiling Specs", MarkSlow, func() {
 	Describe("Measuring code coverage", func() {
 		BeforeEach(func() {
 			fm.MountFixture("coverage")
@@ -141,6 +141,12 @@ var _ = Describe("Profiling Specs", func() {
 				Ω(fm.PathTo("coverage", "myprofile.out")).Should(BeAnExistingFile())
 				Ω(fm.PathTo("coverage", "coverprofile.out")).ShouldNot(BeAnExistingFile())
 			})
+
+			It("fails if the cover profile is not just a filename", func() {
+				session := startGinkgo(fm.PathTo("coverage"), "--no-color", "-coverprofile=/path/to/profile.out")
+				Eventually(session).Should(gexec.Exit(1))
+				Ω(session.Err).Should(gbytes.Say(`--coverprofile expects a filename but was given a path: /path/to/profile.out`))
+			})
 		})
 
 		Context("when multiple suites are tested", func() {
@@ -252,13 +258,14 @@ var _ = Describe("Profiling Specs", func() {
 				// The MutexProfile for the lock_contest test should list two functions that wait on a lock.
 				// Unfortunately go doesn't seem to capture the names of the functions - so they're listed here as
 				// lock_contest_test.glob..func1.1 is called 10 times and takes ~5ms per call
-				// lock_contest_test.glob..func2.1 is called once and teakes ~500ms per call
+				// lock_contest_test.glob..func2.1 is called once and takes ~500ms per call
+				// Note that in Go 1.22 and later, the functions are called lock_contest_test.init.func1.1 and lock_contest_test.init.func2.1
 				// Asserting that both times are within a range should be stable across tests.  The function names should be as well
 				// but that might become a source of failure in the future
 				// Note: these numbers are adjusted slightly to tolerate variance during test runs
-				Ω(mutexProfile.FindCaller("lock_contest_test.glob..func1.1").CumStat).Should(BeNumerically(">=", 45))
-				Ω(mutexProfile.FindCaller("lock_contest_test.glob..func1.1").CumStat).Should(BeNumerically("<", 500))
-				Ω(mutexProfile.FindCaller("lock_contest_test.glob..func2.1").CumStat).Should(BeNumerically(">=", 450))
+				Ω(mutexProfile.FindCaller(fmt.Sprintf("%sfunc1.1", lockContestPrefix)).CumStat).Should(BeNumerically(">=", 45))
+				Ω(mutexProfile.FindCaller(fmt.Sprintf("%sfunc1.1", lockContestPrefix)).CumStat).Should(BeNumerically("<", 500))
+				Ω(mutexProfile.FindCaller(fmt.Sprintf("%sfunc2.1", lockContestPrefix)).CumStat).Should(BeNumerically(">=", 450))
 			},
 
 			Entry("when running in series",
@@ -468,6 +475,7 @@ var _ = Describe("Profiling Specs", func() {
 				"--mutexprofile=mutex.out",
 				"--coverprofile=cover.out",
 				"--json-report=report.json",
+				"--gojson-report=report.go.json",
 				"--junit-report=report.xml",
 				"--teamcity-report=report.tm",
 				"--procs=2",
@@ -479,6 +487,7 @@ var _ = Describe("Profiling Specs", func() {
 			Ω(fm.ListDir("output")).Should(ConsistOf(
 				"cover.out",
 				"report.json",
+				"report.go.json",
 				"report.xml",
 				"report.tm",
 				"block_contest_cpu.out",
@@ -508,6 +517,7 @@ var _ = Describe("Profiling Specs", func() {
 				"--mutexprofile=mutex.out",
 				"--coverprofile=cover.out",
 				"--json-report=report.json",
+				"--gojson-report=report.go.json",
 				"--junit-report=report.xml",
 				"--teamcity-report=report.tm",
 				"--procs=2",
@@ -525,6 +535,9 @@ var _ = Describe("Profiling Specs", func() {
 				"block_contest_report.json",
 				"lock_contest_report.json",
 				"slow_memory_hog_report.json",
+				"block_contest_report.go.json",
+				"lock_contest_report.go.json",
+				"slow_memory_hog_report.go.json",
 				"block_contest_report.xml",
 				"lock_contest_report.xml",
 				"slow_memory_hog_report.xml",
