@@ -338,16 +338,17 @@ var _ = Describe("Table driven tests", func() {
 			})
 		})
 
-		DescribeTable("it works when nils are passed in", func(a interface{}, b error) {
+		DescribeTable("it works when nils are passed in", func(a any, b error) {
 			Ω(a).Should(BeNil())
 			Ω(b).Should(BeNil())
 		}, Entry("nils", nil, nil))
 
-		DescribeTable("it supports variadic parameters", func(a int, b string, c ...interface{}) {
+		DescribeTable("it supports variadic parameters", func(a int, b string, c ...any) {
 			Ω(a).Should(Equal(c[0]))
 			Ω(b).Should(Equal(c[1]))
 			Ω(c[2]).Should(BeNil())
 		}, Entry("variadic arguments", 1, "one", 1, "one", nil))
+
 	})
 
 	Describe("when table entries are marked pending", func() {
@@ -474,7 +475,7 @@ var _ = Describe("Table driven tests", func() {
 					Entry("A", 1),
 					Entry("B", 2),
 					Entry("C", 3),
-					Entry("D", []interface{}{FlakeAttempts(3), Offset(2)}, 3),
+					Entry("D", []any{FlakeAttempts(3), Offset(2)}, 3),
 				)
 			})
 			Ω(success).Should(BeFalse())
@@ -516,7 +517,7 @@ var _ = Describe("Table driven tests", func() {
 					Entry("A", 1),
 					Entry("B", 2),
 					Entry("C", 3),
-					Entry("D", []interface{}{MustPassRepeatedly(3), Offset(2)}, 3),
+					Entry("D", []any{MustPassRepeatedly(3), Offset(2)}, 3),
 				)
 			})
 			Ω(success).Should(BeFalse())
