@@ -2,11 +2,15 @@ package internal_test
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"os"
 	"reflect"
+	"runtime"
+	"slices"
 	"time"
 
+	"github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/ginkgo/v2"
 	"github.com/onsi/ginkgo/v2/types"
 	. "github.com/onsi/gomega"
@@ -43,21 +47,22 @@ var _ = Describe("Partitioning Decorations", func() {
 			SpecTimeout(time.Second),
 			nil,
 			1,
-			[]interface{}{Focus, Pending, []interface{}{Offset(2), Serial, FlakeAttempts(2)}, Ordered, Label("a", "b", "c"), NodeTimeout(time.Second)},
-			[]interface{}{1, 2, 3.1, nil},
+			[]any{Focus, Pending, []any{Offset(2), Serial, FlakeAttempts(2)}, Ordered, Label("a", "b", "c"), NodeTimeout(time.Second)},
+			[]any{1, 2, 3.1, nil},
 			PollProgressInterval(time.Second),
 			PollProgressAfter(time.Second),
 			[]string{"a", "b", "c"},
 			Label("A", "B", "C"),
 			Label("D"),
-			[]interface{}{},
+			[]any{},
 			FlakeAttempts(1),
 			MustPassRepeatedly(1),
+			SpecPriority(2),
 			true,
 			OncePerOrdered,
 		)
 
-		Ω(decorations).Should(Equal([]interface{}{
+		Ω(decorations).Should(Equal([]any{
 			Offset(3),
 			types.NewCustomCodeLocation("hey there"),
 			Focus,
@@ -69,25 +74,26 @@ var _ = Describe("Partitioning Decorations", func() {
 			NodeTimeout(time.Second),
 			GracePeriod(time.Second),
 			SpecTimeout(time.Second),
-			[]interface{}{Focus, Pending, []interface{}{Offset(2), Serial, FlakeAttempts(2)}, Ordered, Label("a", "b", "c"), NodeTimeout(time.Second)},
+			[]any{Focus, Pending, []any{Offset(2), Serial, FlakeAttempts(2)}, Ordered, Label("a", "b", "c"), NodeTimeout(time.Second)},
 			PollProgressInterval(time.Second),
 			PollProgressAfter(time.Second),
 			Label("A", "B", "C"),
 			Label("D"),
 			FlakeAttempts(1),
 			MustPassRepeatedly(1),
+			SpecPriority(2),
 			OncePerOrdered,
 		}))
 
-		Ω(remaining).Should(Equal([]interface{}{
+		Ω(remaining).Should(Equal([]any{
 			Foo{3},
 			"hey there",
 			2.0,
 			nil,
 			1,
-			[]interface{}{1, 2, 3.1, nil},
+			[]any{1, 2, 3.1, nil},
 			[]string{"a", "b", "c"},
-			[]interface{}{},
+			[]any{},
 			true,
 		}))
 	})
@@ -99,6 +105,12 @@ var _ = Describe("Combining Labels", func() {
 	})
 })
 
+var _ = Describe("Combining SemVerConstraints", func() {
+	It("can combine semantic version constraints and produce the unique union", func() {
+		Ω(internal.UnionOfSemVerConstraints(SemVerConstraint("> 2.1.0", "< 2.2.0"), SemVerConstraint("> 2.1.0", "< 2.3.0"))).Should(Equal(SemVerConstraint("> 2.1.0", "< 2.2.0", "< 2.3.0")))
+	})
+})
+
 var _ = Describe("Constructing nodes", func() {
 	var dt *types.DeprecationTracker
 	var didRun bool
@@ -426,7 +438,7 @@ var _ = Describe("Constructing nodes", func() {
 		})
 
 		It("appends and dedupes all labels together, even if nested", func() {
-			node, errors := internal.NewNode(dt, ntIt, "text", body, Label("A", "B", "C"), Label("D", "E", "C"), []interface{}{Label("F"), []interface{}{Label("G", "H", "A", "F")}})
+			node, errors := internal.NewNode(dt, ntIt, "text", body, Label("A", "B", "C"), Label("D", "E", "C"), []any{Label("F"), []any{Label("G", "H", "A", "F")}})
 			Ω(node.Labels).Should(Equal(Labels{"A", "B", "C", "D", "E", "F", "G", "H"}))
 			ExpectAllWell(errors)
 		})
@@ -452,6 +464,78 @@ var _ = Describe("Constructing nodes", func() {
 		})
 	})
 
+	Describe("The SemVerConstraint decoration", func() {
+		It("has no SemVerConstraints by default", func() {
+			node, errors := internal.NewNode(dt, ntIt, "text", body)
+			Ω(node).ShouldNot(BeZero())
+			Ω(node.SemVerConstraints).Should(Equal(SemVerConstraints{}))
+			ExpectAllWell(errors)
+		})
+
+		It("can track SemVerConstraints", func() {
+			node, errors := internal.NewNode(dt, ntIt, "text", body, SemVerConstraint(">= 1.0.0", "< 2.0.0"))
+			Ω(node.SemVerConstraints).Should(Equal(SemVerConstraints{">= 1.0.0", "< 2.0.0"}))
+			ExpectAllWell(errors)
+		})
+
+		It("appends and dedupes all SemVerConstraints together, even if nested", func() {
+			node, errors := internal.NewNode(dt, ntIt, "text", body, SemVerConstraint(">= 1.0.0"), SemVerConstraint("< 2.0.0"), []any{SemVerConstraint(">= 1.0.0"), []any{SemVerConstraint("< 1.9.0")}})
+			Ω(node.SemVerConstraints).Should(Equal(SemVerConstraints{">= 1.0.0", "< 2.0.0", "< 1.9.0"}))
+			ExpectAllWell(errors)
+		})
+
+		It("can be applied to containers", func() {
+			node, errors := internal.NewNode(dt, ntCon, "text", body, SemVerConstraint(">= 1.0.0", "< 2.0.0"))
+			Ω(node.SemVerConstraints).Should(Equal(SemVerConstraints{">= 1.0.0", "< 2.0.0"}))
+			ExpectAllWell(errors)
+		})
+
+		It("cannot be applied to non-container/it nodes", func() {
+			node, errors := internal.NewNode(dt, ntBef, "", body, cl, SemVerConstraint(">= 1.0.0", "< 2.0.0"))
+			Ω(node).Should(BeZero())
+			Ω(errors).Should(ConsistOf(types.GinkgoErrors.InvalidDecoratorForNodeType(cl, ntBef, "SemVerConstraint")))
+			Ω(dt.DidTrackDeprecations()).Should(BeFalse())
+		})
+
+		It("validates SemVerConstraints", func() {
+			node, errors := internal.NewNode(dt, ntIt, "", body, cl, SemVerConstraint("&| 1.0.0", ""))
+			Ω(node).Should(BeZero())
+			Ω(errors).Should(ConsistOf(types.GinkgoErrors.InvalidSemVerConstraint("&| 1.0.0", "improper constraint: &| 1.0.0", cl), types.GinkgoErrors.InvalidEmptySemVerConstraint(cl)))
+			Ω(dt.DidTrackDeprecations()).Should(BeFalse())
+		})
+	})
+
+	Describe("The SpecPriority decorator", func() {
+		It("has no SpecPriority by default", func() {
+			node, errors := internal.NewNode(dt, ntIt, "text", body)
+			Ω(node).ShouldNot(BeZero())
+			Ω(node.SpecPriority).Should(Equal(0))
+			Ω(node.HasExplicitlySetSpecPriority).Should(BeFalse())
+			ExpectAllWell(errors)
+		})
+
+		It("can be applied to containers", func() {
+			node, errors := internal.NewNode(dt, ntCon, "text", body, SpecPriority(5))
+			Ω(node.SpecPriority).Should(Equal(5))
+			Ω(node.HasExplicitlySetSpecPriority).Should(BeTrue())
+			ExpectAllWell(errors)
+		})
+
+		It("can be applied to its", func() {
+			node, errors := internal.NewNode(dt, ntIt, "text", body, SpecPriority(5))
+			Ω(node.SpecPriority).Should(Equal(5))
+			Ω(node.HasExplicitlySetSpecPriority).Should(BeTrue())
+			ExpectAllWell(errors)
+		})
+
+		It("cannot be applied to non-container/it nodes", func() {
+			node, errors := internal.NewNode(dt, ntBef, "", body, cl, SpecPriority(5))
+			Ω(node).Should(BeZero())
+			Ω(errors).Should(ConsistOf(types.GinkgoErrors.InvalidDecoratorForNodeType(cl, ntBef, "SpecPriority")))
+			Ω(dt.DidTrackDeprecations()).Should(BeFalse())
+		})
+	})
+
 	Describe("the timeout-related decorators", func() {
 		It("correctly assigned timeouts when specified", func() {
 			node, errors := internal.NewNode(dt, ntIt, "spec", func(_ SpecContext) {}, cl, NodeTimeout(time.Second), SpecTimeout(2*time.Second), GracePeriod(3*time.Second))
@@ -482,7 +566,7 @@ var _ = Describe("Constructing nodes", func() {
 		})
 
 		It("fails if a timeout is applied to a function that does not take a context", func() {
-			for _, decorator := range []interface{}{NodeTimeout(time.Second), SpecTimeout(time.Second), GracePeriod(time.Second)} {
+			for _, decorator := range []any{NodeTimeout(time.Second), SpecTimeout(time.Second), GracePeriod(time.Second)} {
 				dt = types.NewDeprecationTracker()
 				_, errors := internal.NewNode(dt, ntIt, "spec", func(_ SpecContext) {}, cl, decorator)
 				Ω(errors).Should(BeEmpty())
@@ -593,7 +677,7 @@ var _ = Describe("Constructing nodes", func() {
 		})
 
 		It("errors if the function takes one argument and that argument is not the deprecated Done channel, or a context", func() {
-			f := func(chan interface{}) {}
+			f := func(chan any) {}
 			node, errors := internal.NewNode(dt, ntIt, "text", f, cl)
 			Ω(node).Should(BeZero())
 			Ω(errors).Should(ConsistOf(types.GinkgoErrors.InvalidBodyType(reflect.TypeOf(f), cl, ntIt)))
@@ -644,7 +728,7 @@ var _ = Describe("Constructing nodes", func() {
 
 	Describe("when decorations are nested in slices", func() {
 		It("unrolls them first", func() {
-			node, errors := internal.NewNode(dt, ntIt, "text", []interface{}{body, []interface{}{Focus, FlakeAttempts(3), Label("A")}, FlakeAttempts(2), Label("B"), Label("C", "D")})
+			node, errors := internal.NewNode(dt, ntIt, "text", []any{body, []any{Focus, FlakeAttempts(3), Label("A")}, FlakeAttempts(2), Label("B"), Label("C", "D")})
 			Ω(node.FlakeAttempts).Should(Equal(2))
 			Ω(node.MarkedFocus).Should(BeTrue())
 			Ω(node.Labels).Should(Equal(Labels{"A", "B", "C", "D"}))
@@ -656,7 +740,6 @@ var _ = Describe("Constructing nodes", func() {
 })
 
 var _ = Describe("Node", func() {
-	// HERE - and all the fun edge cases
 	Describe("The nodes that take more specific functions", func() {
 		var dt *types.DeprecationTracker
 		BeforeEach(func() {
@@ -1465,6 +1548,27 @@ var _ = Describe("Nodes", func() {
 		})
 	})
 
+	Describe("SemVerConstraints and UnionOfSemVerConstraints", func() {
+		var nodes Nodes
+		BeforeEach(func() {
+			nodes = Nodes{N(SemVerConstraint(">= 1.0.0", "< 2.0.0")), N(SemVerConstraint("^1.2.3")), N(), N(SemVerConstraint(">= 1.0.0")), N(SemVerConstraint("~1.2.x"))}
+		})
+
+		It("SemVerConstraints returns a slice containing the SemVerConstraints for each node in order", func() {
+			Ω(nodes.SemVerConstraints()).Should(Equal([][]string{
+				{">= 1.0.0", "< 2.0.0"},
+				{"^1.2.3"},
+				{},
+				{">= 1.0.0"},
+				{"~1.2.x"},
+			}))
+		})
+
+		It("UnionOfSemVerConstraints returns a single slice of SemVerConstraints harvested from all nodes and deduped", func() {
+			Ω(nodes.UnionOfSemVerConstraints()).Should(Equal([]string{">= 1.0.0", "< 2.0.0", "^1.2.3", "~1.2.x"}))
+		})
+	})
+
 	Describe("CodeLocation", func() {
 		var nodes Nodes
 		var cl1, cl2 types.CodeLocation
@@ -1479,6 +1583,36 @@ var _ = Describe("Nodes", func() {
 		})
 	})
 
+	Describe("Computing SpecPriority", func() {
+		Context("when no spec priority is set", func() {
+			It("returns zero", func() {
+				nodes := Nodes{N(ntCon), N(ntCon), N(ntIt)}
+				Ω(nodes.GetSpecPriority()).Should(Equal(0))
+			})
+		})
+
+		Context("when spec priority is set on the leaf spec node", func() {
+			It("chooses that spec priority", func() {
+				nodes := Nodes{N(ntCon), N(ntCon), N(ntIt, SpecPriority(42))}
+				Ω(nodes.GetSpecPriority()).Should(Equal(42))
+			})
+		})
+
+		Context("when spec priority is set on an earlier container node", func() {
+			It("chooses that pec priority", func() {
+				nodes := Nodes{N(ntCon, SpecPriority(-17)), N(ntCon), N(ntIt)}
+				Ω(nodes.GetSpecPriority()).Should(Equal(-17))
+			})
+		})
+
+		Context("when spec priority is set on multiple nodes", func() {
+			It("chooses the inner-most priority", func() {
+				nodes := Nodes{N(ntCon, SpecPriority(-17)), N(ntCon, SpecPriority(42)), N(ntIt, SpecPriority(1))}
+				Ω(nodes.GetSpecPriority()).Should(Equal(1))
+			})
+		})
+	})
+
 	Describe("BestTextFor", func() {
 		var nIt, nBef1, nBef2 Node
 		var nodes Nodes
@@ -1652,7 +1786,6 @@ var _ = Describe("Nodes", func() {
 				Ω(nodes.GetMaxMustPassRepeatedly()).Should(Equal(2))
 			})
 		})
-
 	})
 
 	Describe("Labels", func() {
@@ -1670,6 +1803,22 @@ var _ = Describe("Nodes", func() {
 			}).Should(Panic())
 		})
 	})
+
+	Describe("SemVerConstraints", func() {
+		It("can match against a filter", func() {
+			Ω(SemVerConstraint().MatchesSemVerFilter("")).Should(BeTrue())
+			Ω(SemVerConstraint(">= 1.0.0, < 2.0.0").MatchesSemVerFilter("1.2.0")).Should(BeTrue())
+			Ω(SemVerConstraint(">= 1.0.0", "< 2.0.0").MatchesSemVerFilter("1.2.0")).Should(BeTrue())
+			Ω(SemVerConstraint("^1.0.x").MatchesSemVerFilter("1.2.0")).Should(BeTrue())
+			Ω(SemVerConstraint("~1.2.3").MatchesSemVerFilter("1.2.5")).Should(BeTrue())
+			Ω(SemVerConstraint("1.0.0 - 2.0.0").MatchesSemVerFilter("1.2.0")).Should(BeTrue())
+			Ω(SemVerConstraint("!= 1.2.0").MatchesSemVerFilter("1.2.0")).Should(BeFalse())
+			Ω(SemVerConstraint("> 1.2.0").MatchesSemVerFilter("1.2.0")).Should(BeFalse())
+			Ω(func() {
+				SemVerConstraint("> 1.0.0").MatchesSemVerFilter("aaa")
+			}).Should(Panic())
+		})
+	})
 })
 
 var _ = Describe("Iteration Performance", Serial, Label("performance"), func() {
@@ -1818,3 +1967,184 @@ var _ = Describe("Iteration Performance", Serial, Label("performance"), func() {
 		AddReportEntry("Small Slice", gmeasure.RankStats(gmeasure.LowerMedianIsBetter, smallStats...))
 	})
 })
+
+var _ = Describe("NodeArgsTransformers", func() {
+	var dt *types.DeprecationTracker
+	var body func()
+	BeforeEach(func() {
+		dt = types.NewDeprecationTracker()
+		body = func() {}
+	})
+
+	noErrors := func(errs []error) {
+		Ω(errs).To(BeEmpty())
+	}
+
+	panicErrors := func(errs []error) {
+		if len(errs) > 0 {
+			panic(errs)
+		}
+	}
+
+	It("can be nested", func() {
+		removeA := AddTreeConstructionNodeArgsTransformer(func(nodeType types.NodeType, offset Offset, text string, args []any) (string, []any, []error) {
+			return text + " A", append(args, ginkgo.Label("A")), nil
+		})
+		defer removeA()
+		removeB := AddTreeConstructionNodeArgsTransformer(func(nodeType types.NodeType, offset Offset, text string, args []any) (string, []any, []error) {
+			return text + " B", []any{args, ginkgo.Label("B")}, nil
+		})
+		defer removeB()
+		removeC := AddTreeConstructionNodeArgsTransformer(func(nodeType types.NodeType, offset Offset, text string, args []any) (string, []any, []error) {
+			return text + " C", append(args, ginkgo.Label("C")), nil
+		})
+		defer removeC()
+
+		node, errors := internal.NewNode(internal.TransformNewNodeArgs(noErrors, dt, ntIt, "text", body))
+		Ω(errors).To(BeEmpty())
+		Ω(node.Text).To(Equal("text C B A"))
+		Ω(node.Labels).To(ConsistOf("A", "B", "C"))
+
+		removeB()
+		node, errors = internal.NewNode(internal.TransformNewNodeArgs(noErrors, dt, ntIt, "text", body))
+		Ω(errors).To(BeEmpty())
+		Ω(node.Text).To(Equal("text C A"))
+		Ω(node.Labels).To(ConsistOf("A", "C"))
+
+		removeC()
+		removeA()
+		node, errors = internal.NewNode(internal.TransformNewNodeArgs(noErrors, dt, ntIt, "text", body))
+		Ω(errors).To(BeEmpty())
+		Ω(node.Text).To(Equal("text"))
+		Ω(node.Labels).To(BeEmpty())
+	})
+
+	It("check errors", func() {
+		fakeErrors := []error{errors.New("fake error")}
+
+		removeA := AddTreeConstructionNodeArgsTransformer(func(nodeType types.NodeType, offset Offset, text string, args []any) (string, []any, []error) {
+			Fail("should not have been called")
+			return "", nil, nil
+		})
+		defer removeA()
+		removeB := AddTreeConstructionNodeArgsTransformer(func(nodeType types.NodeType, offset Offset, text string, args []any) (string, []any, []error) {
+			return "", nil, fakeErrors
+		})
+		defer removeB()
+
+		defer func() {
+			r := recover()
+			Ω(r).To(Equal(fakeErrors))
+		}()
+
+		internal.NewNode(internal.TransformNewNodeArgs(panicErrors, dt, ntIt, "text", body))
+		Fail("panicErrors should have panicked")
+	})
+
+	It("supports stack unwinding", func() {
+		var caller types.CodeLocation
+		remove := AddTreeConstructionNodeArgsTransformer(func(nodeType types.NodeType, offset Offset, text string, args []any) (string, []any, []error) {
+			caller = types.NewCodeLocation(int(offset))
+			return text, args, nil
+		})
+		defer remove()
+
+		it := func(args ...any) {
+			internal.NewNode(internal.TransformNewNodeArgs(noErrors, dt, ntIt, "text", append(args, body)))
+		}
+
+		_, file, line, _ := runtime.Caller(0)
+		it()
+		Ω(caller).To(Equal(types.CodeLocation{FileName: file, LineNumber: line + 1}))
+
+		helper := func() {
+			it(ginkgo.Offset(1))
+		}
+		_, file, line, _ = runtime.Caller(0)
+		helper()
+		Ω(caller).To(Equal(types.CodeLocation{FileName: file, LineNumber: line + 1}))
+	})
+})
+
+var _, testFileName, describeLine, _ = runtime.Caller(0)
+var _ = Describe("ConstructionNodeReport", func() {
+
+	// expectEqual makes a single assertion at runtime.
+	expectEqual := func(actual, expect types.ConstructionNodeReport) {
+		GinkgoHelper()
+		It("", func() {
+			Ω(actual).To(Equal(expect))
+		})
+	}
+
+	actualDescribeReport := CurrentTreeConstructionNodeReport()
+	expectDescribeReport := newConstructionNodeReport(types.ConstructionNodeReport{}, []container{{"", 0, nil, nil}, {"ConstructionNodeReport", describeLine + 1, []string{}, []string{}}})
+	expectEqual(actualDescribeReport, expectDescribeReport)
+
+	_, _, contextLine, _ := runtime.Caller(0)
+	Context("context", func() {
+		actual := CurrentTreeConstructionNodeReport()
+		expect := newConstructionNodeReport(expectDescribeReport, []container{{"context", contextLine + 1, []string{}, []string{}}})
+		expectEqual(actual, expect)
+	})
+
+	_, _, complexLine, _ := runtime.Caller(0)
+	Context("complex", Label("A"), Label("B"), SemVerConstraint("> 1.0.0", "<= 3.0.0"), func() {
+		actual := CurrentTreeConstructionNodeReport()
+		expect := newConstructionNodeReport(expectDescribeReport, []container{{"complex", complexLine + 1, []string{"A", "B"}, []string{"> 1.0.0", "<= 3.0.0"}}})
+		expectEqual(actual, expect)
+	})
+
+	_, _, serialLine, _ := runtime.Caller(0)
+	Context("serial", Serial, func() {
+		actual := CurrentTreeConstructionNodeReport()
+		expect := expectDescribeReport
+		expect.IsSerial = true
+		expect = newConstructionNodeReport(expect, []container{{"serial", serialLine + 1, []string{"Serial"}, []string{}}})
+		expectEqual(actual, expect)
+	})
+
+	_, _, orderedLine, _ := runtime.Caller(0)
+	Context("ordered", Ordered, func() {
+		actual := CurrentTreeConstructionNodeReport()
+		expect := expectDescribeReport
+		expect.IsInOrderedContainer = true
+		expect = newConstructionNodeReport(expect, []container{{"ordered", orderedLine + 1, []string{}, []string{}}})
+		expectEqual(actual, expect)
+	})
+
+	_, _, outerLine, _ := runtime.Caller(0)
+	Context("outer", func() {
+		Context("inner", func() {
+			actual := CurrentTreeConstructionNodeReport()
+			expect := newConstructionNodeReport(expectDescribeReport, []container{{"outer", outerLine + 1, []string{}, []string{}}, {"inner", outerLine + 2, []string{}, []string{}}})
+			expectEqual(actual, expect)
+		})
+	})
+})
+
+type container struct {
+	text              string
+	line              int
+	labels            []string
+	semVerConstraints []string
+}
+
+// newConstructionNodeReport makes a deep copy and extends the given report.
+func newConstructionNodeReport(report types.ConstructionNodeReport, containers []container) types.ConstructionNodeReport {
+	report.ContainerHierarchyTexts = slices.Clone(report.ContainerHierarchyTexts)
+	report.ContainerHierarchyLocations = slices.Clone(report.ContainerHierarchyLocations)
+	report.ContainerHierarchyLabels = slices.Clone(report.ContainerHierarchyLabels)
+	report.ContainerHierarchySemVerConstraints = slices.Clone(report.ContainerHierarchySemVerConstraints)
+	for _, container := range containers {
+		report.ContainerHierarchyTexts = append(report.ContainerHierarchyTexts, container.text)
+		fileName := ""
+		if container.line != 0 {
+			fileName = testFileName
+		}
+		report.ContainerHierarchyLocations = append(report.ContainerHierarchyLocations, types.CodeLocation{FileName: fileName, LineNumber: container.line})
+		report.ContainerHierarchyLabels = append(report.ContainerHierarchyLabels, container.labels)
+		report.ContainerHierarchySemVerConstraints = append(report.ContainerHierarchySemVerConstraints, container.semVerConstraints)
+	}
+	return report
+}
